         
#include"p30f2011.h"
#include "LCD.h"
#include"L3G4200D.h"
#include "main.h"


_FOSC(CSW_FSCM_OFF & XT_PLL8);	//external OSC 8xPLL 4MHz chrystal// 8Mhz FCY
_FWDT(WDT_OFF);                 // Watchdog timer off
_FICD(PGD2);					//Set Debug PORT
_FGS(CODE_PROT_OFF);  			//CODE protect off


		


unsigned char DataAvaible = 0;	
char Number[7];
double Degrees[] = {0,0,0};
double Rate[3];
double PrevRate[3];
unsigned int PrevClock = 0;
const char ZeroPoint [] = {67, -42, -13};

int main ()
{
	
	ADPCFG = 65535;		//Disable all Analog pin
	TRISB = 0;			//All IO are output
	TRISC = 0;
	TRISD = 0;
	
	TRISBbits.TRISB6 = 1;	//Interrupt PIN for data avaible on Gyro
	IFS0bits.INT0IF = 0;
	IEC0bits.INT0IE = 1;
	IPC0 = 1;


	T1CON = 0;			//TIMER1
	PR1 = 65535;
	T1CONbits.TON = 1;
	T1CONbits.TCKPS = 0b01;	//8 prescaler, 1uS/bit

	initI2C();
	LCDInit();
	LCDSendCommand(SET_CURSOR);
	GyroInit();

	//Check L3G4200D avaible
	unsigned char data;
	data = readRegister(L3G4200D_ADRESS, L3G_WHO_AM_I);
	if(data == 211)
	{
		LCDSendString("Gyro Init OK");
		__delay_ms(500);
		LCDSendCommand(CLR_DISP);
		
	}
	else
	{
		LCDSendString("Gyro Sensor Init Failed");
		while(1) Nop();
	}

	unsigned int Delay;
	int Data[3];
	while(1)	//Main loop
	{
		if(DataAvaible == 1)	//if data avaible on gyro
		{
			DataAvaible = 0;
			unsigned int Clock = TMR1;
			if(Clock > PrevClock)	//Calculate elapsed time in uS
			{
				Delay = 65535 - PrevClock + Clock;

			}
			else
			{
				Delay = Clock - PrevClock;
			}
			PrevClock = Clock;

			ReadData(&Data[0]);	//Read gyro data	

			int i;
			for( i = 0; i< 3; i++)
			{
				if(Data[i] <= 150 && Data[i] >= -150)	//Noise check
				{
					Rate[i] = 0;
				}
				else
				{
					Rate[i] = (Data[i]) * 0.0875;
				}
			}
			Degrees[0] += ((Rate[0] + PrevRate[0]) * Delay)/20000000;
			Degrees[1] += ((Rate[1] + PrevRate[1]) * Delay)/20000000;
			Degrees[2] += ((Rate[2] + PrevRate[2]) * Delay)/20000000;
			PrevRate[0] = Rate[0];
			PrevRate[1] = Rate[1];
			PrevRate[2] = Rate[2];

			//Display data on LCD
			IntToChar((int)Degrees[0], &Number[0]);
			LCDSendCommand(SET_CURSOR | 0b00000000);
			LCDSendString("X: ");
			LCDSendString(&Number[0]);	
			LCDSendString("   ");
			IntToChar((int)Degrees[1], &Number[0]);
			LCDSendCommand(SET_CURSOR | 0b00001000);
			LCDSendString("Y: ");
			LCDSendString(&Number[0]);	
			LCDSendString("   ");
			IntToChar((int)Degrees[2], &Number[0]);
			LCDSendCommand(SET_CURSOR | 0b01000000);
			LCDSendString("Z: ");
			LCDSendString(&Number[0]);	
			LCDSendString("   ");
		}		
	}
	
}



void __attribute__((__interrupt__, no_auto_psv)) _INT0Interrupt(void)	//Read Gyro avaible data
{
	DataAvaible = 1;
	IFS0bits.INT0IF = 0;
}



void I2CWait ()
{
	while (I2CCONbits.SEN ==1)
	{
		Nop ();
	}
	while (I2CCONbits.RSEN ==1)
	{
		Nop ();
	}
	while (I2CCONbits.PEN ==1)
	{
		Nop ();
	}
	while (I2CCONbits.ACKEN ==1)
	{
		Nop ();
	}
	while (I2CCONbits.RCEN ==1)
	{
		Nop ();
	}
	while (I2CSTATbits.TRSTAT ==1)
	{
		Nop ();
	}
	if(I2CSTATbits.BCL == 1)
	{
		LCDSendCommand(SET_CURSOR | 0b00000000);
		LCDSendString("I2C Error");
		while(1) Nop();
	}
	
}

void initI2C()	// I2C master mode enable
{
	I2CRCV = 0;
	I2CTRN = 0;
	I2CSTAT = 0;
	I2CCON = 0b1000001000000000;
	I2CBRG = 150;	//100kHz
}

void I2CStart(unsigned char Address)
{
	I2CCONbits.SEN = 1;	 
	I2CWait();

	I2CTRN = Address;
	I2CWait();
}

void I2CReStart(unsigned char Address)
{
	I2CCONbits.RSEN = 1;			 
	I2CWait();
	I2CTRN = Address;
	I2CWait();
}

void I2CSend(unsigned char Data)
{
	I2CTRN = Data;				
	I2CWait();
}


void I2CStop (void)
{
	I2CCONbits.PEN = 1;			
	I2CWait();
}



